/*
 *  MSSQL Communication Library -- API
 *
 *  (c) 1998 Arno Pedusaar <psaar@fenar.ee>
 *  (c) 1998 Martin Mares <mj@ucw.cz>
 */

#ifndef __MSSQL_H__
#define __MSSQL_H__

#ifdef HAVE_SSL
#include <ssl.h>
#endif /* HAVE_SSL */

enum { T_C_STRING=1,T_C_INTEGER,T_C_DOUBLE,T_C_LONGLONG,T_C_TIME,T_C_UNDEFINED };

enum { IO_SOCKET=1,IO_SSL,IO_FILE };

#define ROW_READY 1
#define WAITING_ROW 2
#define DONE 3

enum { BUFFER_EMPTY=0x01,BUFFER_NOT_EMPTY=0x02,BUFFER_UNDERRUN=0x04 };

typedef struct COLUMN
{
	int type;
	unsigned int flags;
	char name[33];
	char tableName[33];

	int len;
	int precision;
	int scale;

	char asString[257];

	void *val;
	int maxlen;
	int boundType;

	struct COLUMN *next;
} COLUMN;

enum { MSG_INFO,MSG_ERROR };

typedef struct MESSAGE
{
	char msg[33];
	int type;
	struct MESSAGE *next;
} MESSAGE;

typedef struct MSSQL
{
	int fd;
	int blockSize;
	unsigned char *outBuf;
	unsigned char *inBuf;
	int outPos;
	int inPos;
	unsigned char outFlag1;
	unsigned char inFlag1;
	unsigned char inState;
	int inFragmentSize;

	int columnCount;
	COLUMN columns;
	COLUMN *currentColumn;

	int queryState;

#ifdef HAVE_SSL
        SSL_CTX*     ssl_ctx;
        SSL*         ssl;
        BIO*         ssl_bio;
        BIO*         bio;
#endif /* HAVE_SSL */

	int (*connect) (struct MSSQL *s,char *name,int port);
	int (*disconnect) (struct MSSQL *s);
	int (*read) (struct MSSQL *s,unsigned char *buf,int len);
	int (*write) (struct MSSQL *s,unsigned char *buf,int len);

	int bytesIn;
	int bytesOut;
	int verbose;

	MESSAGE msg,*currentMessage;
} MSSQL;

extern MSSQL*  allocMsSql(void);
extern int     connectMsSql(MSSQL *s,char *name,int port);
extern int     disconnectMsSql(MSSQL *s);
extern int doLogin(MSSQL *s,
  char *hostName,
  char *userName,
  char *password,
  char *id,
  char *appName,
  char *serviceName,
  char *language,
  char *charset
  );

extern int bindStringColumn(MSSQL *s,int column,char *buf,int maxlen);
extern int indDoubleColumn(MSSQL *s,int column,double *buf);
extern int bindIntegerColumn(MSSQL *s,int column,int *buf);
extern int bindDoubleColumn(MSSQL *s,int column,double *buf);
extern int bindLonglongColumn(MSSQL *s,int column,long long *buf);
extern int execDirect(MSSQL *s,char *query);
extern int nextTuple(MSSQL *s);
extern int initIo(MSSQL *s,int method);
extern void printStats(MSSQL *s);
extern void printColumnNames(MSSQL *s);
extern void printResult(MSSQL *s);

extern int openLog(MSSQL *s,char *path);
extern void appendLog(MSSQL *s,char *fmt,...);
extern void closeLog(MSSQL *s);

extern void addMessage(MSSQL *s,char *msg,int type);
extern MESSAGE *firstMessage(MSSQL *s);
extern MESSAGE *nextMessage(MSSQL *s);

#endif /* __MSSQL_H__ */
